from datetime import datetime
from zope.interface import implements
from urlparse import urljoin

from twisted.plugin import IPlugin
from twisted.web.client import getPage
from twisted.internet import defer
from twisted.python import log

from malparser import MAL
from malparser.anime import Anime

from apiserver import settings
from apiserver.interfaces import IMetadata
from apiserver.metadata._simplemetadatahandler import SimpleMetadata
from apiserver.db import metadata_db


class TwistedMAL(MAL):
    @defer.inlineCallbacks
    def _fetch(self, obj):
        data = yield getPage(str(obj._get_url()))
        obj.parse(data)

class MALMetadata(SimpleMetadata):
    implements(IPlugin, IMetadata)
    currently_fetching = False
    id = 'mal'
    
    trigger = 'plugin:mal:id'
    
    def __init__(self):
        self._mal = TwistedMAL()
    
    @defer.inlineCallbacks
    def fetch_metadata(self, obj_id):
        anime = self._mal.get_anime(obj_id)
        yield anime.fetch()
        
        image_file = yield self.download_cover(obj_id, anime.cover)
        
        anime_info = {
            'last_updated': datetime.now(),
            'title': anime.title,
            'duration': anime.info['Duration'],
            'episodes': anime.info['Episodes'],
            'genres': [genre['name'] for genre in anime.info['Genres']],
            'producers': [producer['name'] for producer in anime.info['Producers']],
            'tv_rating': anime.info['Rating'],
            'type': anime.info['Type'],
            'popularity': anime.statistics['Popularity'],
            'ranked': anime.statistics['Ranked'],
            'rating': anime.statistics['Score'],
            'votes': anime.statistics['Votes'],
            'plot': anime.synopsis,
            'cover': image_file,
            'alternative_titles': anime.alternative_titles,
            'season': anime.aired.get('Season', None),
            'aired_start': anime.aired.get('Aired_start', None),
            'aired_end': anime.aired.get('Aired_end', None),
            'related': dict([(k, v) for k, v in [(k, [a.mal_id for a in v if isinstance(a, Anime)]) for k, v in anime.related.items()] if v]), # extracts all anime ids and removes empty keys
        }
        
        defer.returnValue(anime_info)
    
    def actual_handle_item(self, user, item, raw_item, obj):
        item['premiered'] = item['aired_start'] = item.get('aired_start', None) and item['aired_start'].isoformat() or None
        item['aired_end'] = item.get('aired_end', None) and item['aired_end'].isoformat() or None
        
        del item['last_updated']
        
        if item['cover']:
            item['cover'] = urljoin(settings.BASE_URL, 'files/mal/%s' % obj['cover'])
        
        return item

malmetadata = MALMetadata()